﻿using Dapper;
using Microsoft.Data.SqlClient;
using Microsoft.Extensions.Configuration;
using Shared.DTO.Integracion_DGA;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DAL
{
  public class JobsDgaSupFlujRepository 
  {
    private static string connectionString = string.Empty;
    private readonly IConfiguration _configuration;

    public JobsDgaSupFlujRepository(IConfiguration configuration)
    {
      _configuration = configuration;
      connectionString = _configuration.GetConnectionString("DefaultConnection") ?? "";
    }

    public async Task<bool> InsertarDgaMacroResultadoSupFluj(List<DgaMacroResultadoSupFluj> dgaMacroResultadoSupFluj)
    {
      try
      {        
        // Configurar la conexión a la base de datos
        using (SqlConnection connection = new SqlConnection(connectionString))
        {
          await connection.OpenAsync();
          await connection.ExecuteAsync("TRUNCATE TABLE DGA_MACRO_RESULTADO_SUP_FLUJ");

          await SpCalculoDgaSupFluj();

          string sql = "INSERT INTO DGA_MACRO_RESULTADO_SUP_FLUJ (TagName, TimeStamp, Value, Quality) VALUES (@TagName, @TimeStamp, @Value, @Quality)";
          await connection.ExecuteAsync(sql, dgaMacroResultadoSupFluj);
          return true;
        }
      }
      catch (Exception ex)
      {
        throw new Exception($"Error: {ex.Message}");
      }
    }

    public async Task<bool> SpCalculoDgaSupFluj()
    {
        try
        {
            using (var connection = new SqlConnection(connectionString))
            {
                await connection.OpenAsync();
                await connection.ExecuteAsync(
                    "SP_CALCULO_DGA_SUP_FLUJ",
                    commandType: CommandType.StoredProcedure
                );
            }
            return true; // Éxito
        }
        catch (Exception ex)
        {
            throw new Exception($"Error: {ex.Message}");
        }
    }

    public async Task<bool> SpTraspasoDatosAchirdSupFluj()
    {
        try
        {
            using (var connection = new SqlConnection(connectionString))
            {
                await connection.OpenAsync();
                await connection.ExecuteAsync(
                    "SP_TRASPASO_DATOS_ACHIRD_SUP_FLUJ",
                    commandType: CommandType.StoredProcedure
                );
            }
            return true; // Éxito
        }
        catch (Exception ex)
        {
            throw new Exception($"Error: {ex.Message}");
        }
    }
    }
}
